@@ -1,20 +1,43 @@
-BRISC_estimation <- function(coords, x, y, sigma.sq = 1, tau.sq = 0.1, phi = 1, nu = 0.5, n.neighbors = 15, n_omp = 1,
-                             order = "Sum_coords", cov.model = "exponential", search.type = "tree", verbose = TRUE, eps = 2e-05, nugget_status = 1
+BRISC_estimation <- function(coords, y, x = NULL, sigma.sq = 1, tau.sq = 0.1, phi = 1, nu = 1.5, n.neighbors = 15, n_omp = 1,
+                             order = "Sum_coords", cov.model = "exponential", search.type = "tree", stabilization = NULL,
+                             pred.stabilization = 1e-8, verbose = TRUE, eps = 2e-05, nugget_status = 1, tol = 12
 ){
+  n <- nrow(coords)
+  if(is.null(x)){
+    x <- matrix(1, nrow = n, ncol = 1)
+  }
   p <- ncol(x)
-  n <- nrow(x)
+  if(ncol(x) == 1) warning('The ordering of inputs x (covariates) and y (response) in BRISC_estimation has been changed BRISC 1.0.0 onwards.
+Please check the new documentation with ?BRISC_estimation.')
   ##Coords and ordering
   if(nugget_status == 0){fix_nugget = 0}
   if(nugget_status == 1){fix_nugget = 1}
   if(!is.matrix(coords)){stop("error: coords must n-by-2 matrix of xy-coordinate locations")}
   if(order == "AMMD" && length(y) < 65){stop("error: Number of data points must be atleast 65 to use AMMD")}
   if(ncol(coords) != 2 || nrow(coords) != n){
-    stop("error: either the coords have more than two columns or then number of rows is different than
+    stop("error: either the coords have more than two columns or the number of rows is different than
          data used in the model formula")
   }
+  if(is.matrix(y) & ncol(as.matrix(y)) > 1)
+    {stop("error: BRISC handles only univariate response. You have used a y (response) with more than one column.
+          The ordering of inputs x (covariates) and y (response) in BRISC_estimation has been changed BRISC 0.2.0 onwards.
+          Did you mean x (covaiates) instead of y (response) by the multivariatte input?
+          Please check the new documentation with ?BRISC_estimation.")
+
+  }
+
+  coords <- round(coords, tol)
+  x <- round(x, tol)
+  y <- round(y, tol)
   if(order != "AMMD" && order != "Sum_coords"){
     stop("error: Please insert a valid ordering scheme choice given by 1 or 2.")
   }
+
+  if(tau.sq < 0 ){stop("error: tau.sq must be non-negative")}
+  if(sigma.sq < 0 ){stop("error: sigma.sq must be non-negative")}
+  if(phi < 0 ){stop("error: phi must be non-negative")}
+  if(nu < 0 ){stop("error: nu must be non-negative")}
+
   if(verbose == TRUE){
     cat(paste(("----------------------------------------"), collapse="   "), "\n"); cat(paste(("\tOrdering Coordinates"), collapse="   "), "\n")
     }
@@ -34,6 +57,32 @@ BRISC_estimation <- function(coords, x, y, sigma.sq = 1, tau.sq = 0.1, phi = 1,
   storage.mode(cov.model.indx) <- "integer"
 
 
+  ##Stabilization
+  if(is.null(stabilization)){
+    if(cov.model == "exponential"){
+      stabilization = FALSE
+    }
+    if(cov.model != "exponential"){
+      stabilization = TRUE
+    }
+  }
+
+  if(!isTRUE(stabilization)){
+    if(cov.model != "exponential"){
+      warning('We recommend using stabilization for spherical, Matern and Gaussian covariance model')
+    }
+  }
+
+  if(isTRUE(stabilization)){
+    taus <- 10^-6 * sigma.sq
+    artificial_noise <- sqrt(taus) * rnorm(n)
+    y <- y + artificial_noise
+    tau.sq <- tau.sq + taus
+    fix_nugget <- 1
+    if(verbose == TRUE) cat(paste(("----------------------------------------"), collapse="   "), "\n"); cat(paste(("\tUsing Stabilization"), collapse="   "), "\n"); cat(paste(("----------------------------------------"), collapse="   "), "\n")
+  }
+
+
   ##Initial values
   if(cov.model!="matern"){
     initiate <- c(sigma.sq, tau.sq, phi)
@@ -53,7 +102,7 @@ BRISC_estimation <- function(coords, x, y, sigma.sq = 1, tau.sq = 0.1, phi = 1,
 
 
   ##Search type
-  search.type.names <- c("brute", "tree")
+  search.type.names <- c("brute", "tree", "cb")
   if(!search.type %in% search.type.names){
     stop("error: specified search.type '",search.type,"' is not a valid option; choose from ", paste(search.type.names, collapse=", ", sep="") ,".")
   }
@@ -80,15 +129,18 @@ BRISC_estimation <- function(coords, x, y, sigma.sq = 1, tau.sq = 0.1, phi = 1,
 
   p1<- proc.time()
 
-  if(nugget_status == 2){alpha.sq.starting = 0}
+  if(nugget_status == 0){alpha.sq.starting = 0}
 
 
   ##estimtion
-  result <- .Call("BRISC_estimatecpp", y, X, p, n, n.neighbors, coords, cov.model.indx, alpha.sq.starting, phi.starting, nu.starting, search.type.indx, n.omp.threads, verbose, eps, fix_nugget)
+  result <- .Call("BRISC_estimatecpp", y, X, p, n, n.neighbors, coords, cov.model.indx, alpha.sq.starting, phi.starting, nu.starting, search.type.indx, n.omp.threads, verbose, eps, fix_nugget, PACKAGE = "BRISC")
 
   p2 <- proc.time()
 
   Theta <- result$theta
+  if(!is.null(pred.stabilization)){
+    Theta[2] <- max(result$theta[1] * pred.stabilization, result$theta[2])
+  }
   if(cov.model!="matern"){
     names(Theta) <- c("sigma.sq", "tau.sq", "phi")
   }
@@ -109,10 +161,10 @@ BRISC_estimation <- function(coords, x, y, sigma.sq = 1, tau.sq = 0.1, phi = 1,
   result_list$cov.model <- cov.model
   result_list$eps <- eps
   result_list$init <- initiate
-  result_list$Beta = Beta
-  result_list$Theta = Theta
-  result_list$estimation.time = p2 - p1
-  result_list$BRISC_Object = result
+  result_list$Beta <- Beta
+  result_list$Theta <- Theta
+  result_list$estimation.time <- p2 - p1
+  result_list$BRISC_Object <- result
   class(result_list) <- "BRISC_Out"
   result_list
 }
