@@ -95,7 +95,7 @@ extern "C" {
                 F[i] = 1 - F77_NAME(ddot)(&nnIndxLU[n+i], &B[nnIndxLU[i]], &inc, &c[nnIndxLU[i]], &inc) + theta[0]*fix_nugget;
             }else{
                 B[i] = 0;
-                F[i] = 1 + theta[0]*fix_nugget;
+                F[i] = 1+ theta[0]*fix_nugget;
             }
         }
         for(i = 0; i < n; i++){
@@ -319,10 +319,10 @@ extern "C" {
     void processed_bootstrap_output(double *X, double *y_boot, double *D, double *d, int *nnIndx, int *nnIndxLU, int *CIndx, int n, int p, int m, double *theta, int covModel, int j, int nThreads, double optimized_likelihod, double *beta_boot, double *theta_fp_boot, double fix_nugget){
 
         int nIndx = static_cast<int>(static_cast<double>(1+m)/2*m+(n-m-1)*m);
-        double *B = (double *) R_alloc(nIndx, sizeof(double));
-        double *F = (double *) R_alloc(n, sizeof(double));
-        double *c =(double *) R_alloc(nIndx, sizeof(double));
-        double *C = (double *) R_alloc(j, sizeof(double)); zeros(C, j);
+        double *B = (double *) calloc(nIndx, sizeof(double));
+        double *F = (double *) calloc(n, sizeof(double));
+        double *c =(double *) calloc(nIndx, sizeof(double));
+        double *C = (double *) calloc(j, sizeof(double)); zeros(C, j);
 
         double logDet;
 
@@ -335,9 +335,9 @@ extern "C" {
         char const *lower = "L";
 
 
-        double *tmp_pp = (double *) R_alloc(pp, sizeof(double));
-        double *tmp_p = (double *) R_alloc(p, sizeof(double));
-        double *tmp_n = (double *) R_alloc(n, sizeof(double));
+        double *tmp_pp = (double *) calloc(pp, sizeof(double));
+        double *tmp_p = (double *) calloc(p, sizeof(double));
+        double *tmp_n = (double *) calloc(n, sizeof(double));
 
         //create B and F
         logDet = updateBF(B, F, c, C, D, d, nnIndxLU, CIndx, n, theta, covModel, nThreads, fix_nugget);
@@ -378,6 +378,14 @@ extern "C" {
         if (covModel == 2) {
             theta_fp_boot[3] = theta[2];
         }
+
+        free(B);
+        free(F);
+        free(c);
+        free(C);
+        free(tmp_pp);
+        free(tmp_p);
+        free(tmp_n);
     }
 
     SEXP process_bootstrap_data(SEXP B_r, SEXP F_r, SEXP Xbeta_r, SEXP norm_residual_boot_r, SEXP nnIndx_r, SEXP nnIndxLU_r, SEXP n_r, SEXP p_r){
@@ -600,8 +608,11 @@ extern "C" {
 
         if(INTEGER(sType_r)[0] == 0){
             mkNNIndx(n_nngp, m_nngp, coords, nnIndx_nngp, d_nngp, nnIndxLU_nngp);
-        }else{
+        }
+        if(INTEGER(sType_r)[0] == 1){
             mkNNIndxTree0(n_nngp, m_nngp, coords, nnIndx_nngp, d_nngp, nnIndxLU_nngp);
+        }else{
+            mkNNIndxCB(n_nngp, m_nngp, coords, nnIndx_nngp, d_nngp, nnIndxLU_nngp);
         }
 
 
@@ -748,10 +759,332 @@ extern "C" {
         UNPROTECT(nProtect);
 
 
+        return(result_r);
+    }
+
+    SEXP BRISC_correlationcpp(SEXP n_r, SEXP m_r, SEXP coords_r, SEXP covModel_r, SEXP alphaSqStarting_r, SEXP phiStarting_r, SEXP nuStarting_r,
+                              SEXP sType_r, SEXP nThreads_r, SEXP verbose_r, SEXP sim_r, SEXP sim_number_r, SEXP fix_nugget_r){
+
+        int i, j, k, l, nProtect=0;
+
+        //get args
+        int n = INTEGER(n_r)[0];
+        int m = INTEGER(m_r)[0];
+        double fix_nugget = REAL(fix_nugget_r)[0];
+        double *coords = REAL(coords_r);
+
+        int covModel = INTEGER(covModel_r)[0];
+        std::string corName = getCorName(covModel);
+
+        int nThreads = INTEGER(nThreads_r)[0];
+        int sim_number = INTEGER(sim_number_r)[0];
+        double *sim = REAL(sim_r);
+        int tot_length = n*sim_number;
+        int verbose = INTEGER(verbose_r)[0];
+
+
+
+#ifdef _OPENMP
+        omp_set_num_threads(nThreads);
+#else
+        if(nThreads > 1){
+            warning("n.omp.threads > %i, but source not compiled with OpenMP support.", nThreads);
+            nThreads = 1;
+        }
+#endif
+
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tModel description\n");
+            Rprintf("----------------------------------------\n");
+            Rprintf("BRISC simulation with %i locations.\n\n", n);
+            Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
+            Rprintf("Using %i nearest neighbors.\n\n", m);
+#ifdef _OPENMP
+            Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n", nThreads);
+#else
+            Rprintf("\n\nSource not compiled with OpenMP support.\n");
+#endif
+        }
+
+        //parameters
+        int nTheta;
+
+        if(corName != "matern"){
+            nTheta = 2;//tau^2 = 0, phi = 1
+        }else{
+            nTheta = 3;//tau^2 = 0, phi = 1, nu = 2;
+        }
+        //starting
+        double *theta = (double *) R_alloc (nTheta, sizeof(double));
+
+        theta[0] = pow(REAL(alphaSqStarting_r)[0], 2.0);
+        theta[1] = pow(REAL(phiStarting_r)[0], 2.0);
+
+        if(corName == "matern"){
+            theta[2] = pow(REAL(nuStarting_r)[0], 2.0);
+        }
+
+        //allocated for the nearest neighbor index vector (note, first location has no neighbors).
+        int nIndx = static_cast<int>(static_cast<double>(1+m)/2*m+(n-m-1)*m);
+
+        int *nnIndx = (int *) R_alloc(nIndx, sizeof(int));
+
+        double *d = (double *) R_alloc(nIndx, sizeof(double));
+
+        int *nnIndxLU = (int *) R_alloc(2*n, sizeof(int));
+        //make the neighbor index
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tBuilding neighbor index\n");
+#ifdef Win32
+            R_FlushConsole();
+#endif
+        }
+
+        if(INTEGER(sType_r)[0] == 0){
+            mkNNIndx(n, m, coords, nnIndx, d, nnIndxLU);
+        }
+        if(INTEGER(sType_r)[0] == 1){
+            mkNNIndxTree0(n, m, coords, nnIndx, d, nnIndxLU);
+        }else{
+            mkNNIndxCB(n, m, coords, nnIndx, d, nnIndxLU);
+        }
+
+
+
+        int *CIndx = (int *) R_alloc(2*n, sizeof(int));
+        for(i = 0, j = 0; i < n; i++){//zero should never be accessed
+            j += nnIndxLU[n+i]*nnIndxLU[n+i];
+            if(i == 0){
+                CIndx[n+i] = 0;
+                CIndx[i] = 0;
+            }else{
+                CIndx[n+i] = nnIndxLU[n+i]*nnIndxLU[n+i];
+                CIndx[i] = CIndx[n+i-1] + CIndx[i-1];
+            }
+        }
+
+
+        double *D = (double *) R_alloc(j, sizeof(double));
+
+        SEXP sim_cor_r; PROTECT(sim_cor_r = allocVector(REALSXP, tot_length)); nProtect++; double *sim_cor = REAL(sim_cor_r);
+
+        for(i = 0; i < n; i++){
+            for(k = 0; k < nnIndxLU[n+i]; k++){
+                for(l = 0; l <= k; l++){
+                    D[CIndx[i]+l*nnIndxLU[n+i]+k] = dist2(coords[nnIndx[nnIndxLU[i]+k]], coords[n+nnIndx[nnIndxLU[i]+k]], coords[nnIndx[nnIndxLU[i]+l]], coords[n+nnIndx[nnIndxLU[i]+l]]);
+                }
+            }
+        }
+
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tCalculationg the approximate Cholesky Decomposition\n");
+#ifdef Win32
+            R_FlushConsole();
+#endif
+        }
+
+        double *B = (double *) R_alloc(nIndx, sizeof(double));
+        double *F = (double *) R_alloc(n, sizeof(double));
+
+
+        double *c =(double *) R_alloc(nIndx, sizeof(double));
+        double *C = (double *) R_alloc(j, sizeof(double)); zeros(C, j);
+
+        double logDet = updateBF(B, F, c, C, D, d, nnIndxLU, CIndx, n, theta, covModel, nThreads, fix_nugget);
+
+        for(int im = 0; im < sim_number; im++){
+            solve_B_F(B, F, &sim[n*im], n, nnIndxLU, nnIndx, &sim_cor[n*im]);
+        }
+
+        //return stuff
+        SEXP result_r, resultName_r;
+        int nResultListObjs = 2;
+
+
+
+        PROTECT(result_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
+        PROTECT(resultName_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
+
+        SET_VECTOR_ELT(result_r, 0, sim_r);
+        SET_VECTOR_ELT(resultName_r, 0, mkChar("norm_sim"));
+
+        SET_VECTOR_ELT(result_r, 1, sim_cor_r);
+        SET_VECTOR_ELT(resultName_r, 1, mkChar("sim"));
+
+        namesgets(result_r, resultName_r);
+
+        //unprotect
+        UNPROTECT(nProtect);
+
+
+        return(result_r);
+    }
+
+
+
+
+
+    SEXP BRISC_decorrelationcpp(SEXP n_r, SEXP m_r, SEXP coords_r, SEXP covModel_r, SEXP alphaSqStarting_r, SEXP phiStarting_r, SEXP nuStarting_r,
+                                SEXP sType_r, SEXP nThreads_r, SEXP verbose_r, SEXP sim_r, SEXP sim_number_r, SEXP fix_nugget_r){
+
+        int i, j, k, l, nProtect=0;
+
+        //get args
+        int n = INTEGER(n_r)[0];
+        int m = INTEGER(m_r)[0];
+        double fix_nugget = REAL(fix_nugget_r)[0];
+        double *coords = REAL(coords_r);
+
+        int covModel = INTEGER(covModel_r)[0];
+        std::string corName = getCorName(covModel);
+
+        int nThreads = INTEGER(nThreads_r)[0];
+        int sim_number = INTEGER(sim_number_r)[0];
+        double *sim = REAL(sim_r);
+        int tot_length = n*sim_number;
+        int verbose = INTEGER(verbose_r)[0];
+
+
+
+#ifdef _OPENMP
+        omp_set_num_threads(nThreads);
+#else
+        if(nThreads > 1){
+            warning("n.omp.threads > %i, but source not compiled with OpenMP support.", nThreads);
+            nThreads = 1;
+        }
+#endif
+
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tModel description\n");
+            Rprintf("----------------------------------------\n");
+            Rprintf("BRISC simulation with %i locations.\n\n", n);
+            Rprintf("Using the %s spatial correlation model.\n\n", corName.c_str());
+            Rprintf("Using %i nearest neighbors.\n\n", m);
+#ifdef _OPENMP
+            Rprintf("\nSource compiled with OpenMP support and model fit using %i thread(s).\n", nThreads);
+#else
+            Rprintf("\n\nSource not compiled with OpenMP support.\n");
+#endif
+        }
+
+        //parameters
+        int nTheta;
+
+        if(corName != "matern"){
+            nTheta = 2;//tau^2 = 0, phi = 1
+        }else{
+            nTheta = 3;//tau^2 = 0, phi = 1, nu = 2;
+        }
+        //starting
+        double *theta = (double *) R_alloc (nTheta, sizeof(double));
+
+        theta[0] = pow(REAL(alphaSqStarting_r)[0], 2.0);
+        theta[1] = pow(REAL(phiStarting_r)[0], 2.0);
+
+        if(corName == "matern"){
+            theta[2] = pow(REAL(nuStarting_r)[0], 2.0);
+        }
+
+        //allocated for the nearest neighbor index vector (note, first location has no neighbors).
+        int nIndx = static_cast<int>(static_cast<double>(1+m)/2*m+(n-m-1)*m);
+
+        int *nnIndx = (int *) R_alloc(nIndx, sizeof(int));
+
+        double *d = (double *) R_alloc(nIndx, sizeof(double));
+
+        int *nnIndxLU = (int *) R_alloc(2*n, sizeof(int));
+
+        //make the neighbor index
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tBuilding neighbor index\n");
+#ifdef Win32
+            R_FlushConsole();
+#endif
+        }
+
+        if(INTEGER(sType_r)[0] == 0){
+            mkNNIndx(n, m, coords, nnIndx, d, nnIndxLU);
+        }
+        if(INTEGER(sType_r)[0] == 1){
+            mkNNIndxTree0(n, m, coords, nnIndx, d, nnIndxLU);
+        }else{
+            mkNNIndxCB(n, m, coords, nnIndx, d, nnIndxLU);
+        }
+
+        int *CIndx = (int *) R_alloc(2*n, sizeof(int));
+        for(i = 0, j = 0; i < n; i++){//zero should never be accessed
+            j += nnIndxLU[n+i]*nnIndxLU[n+i];
+            if(i == 0){
+                CIndx[n+i] = 0;
+                CIndx[i] = 0;
+            }else{
+                CIndx[n+i] = nnIndxLU[n+i]*nnIndxLU[n+i];
+                CIndx[i] = CIndx[n+i-1] + CIndx[i-1];
+            }
+        }
+
+        double *D = (double *) R_alloc(j, sizeof(double));
+
+        SEXP sim_decor_r; PROTECT(sim_decor_r = allocVector(REALSXP, tot_length)); nProtect++; double *sim_decor = REAL(sim_decor_r);
+
+        for(i = 0; i < n; i++){
+            for(k = 0; k < nnIndxLU[n+i]; k++){
+                for(l = 0; l <= k; l++){
+                    D[CIndx[i]+l*nnIndxLU[n+i]+k] = dist2(coords[nnIndx[nnIndxLU[i]+k]], coords[n+nnIndx[nnIndxLU[i]+k]], coords[nnIndx[nnIndxLU[i]+l]], coords[n+nnIndx[nnIndxLU[i]+l]]);
+                }
+            }
+        }
+
+        if(verbose){
+            Rprintf("----------------------------------------\n");
+            Rprintf("\tCalculationg the approximate Cholesky Decomposition\n");
+#ifdef Win32
+            R_FlushConsole();
+#endif
+        }
+
+        double *B = (double *) R_alloc(nIndx, sizeof(double));
+        double *F = (double *) R_alloc(n, sizeof(double));
+
+        double *c =(double *) R_alloc(nIndx, sizeof(double));
+        double *C = (double *) R_alloc(j, sizeof(double)); zeros(C, j);
+
+        double logDet = updateBF(B, F, c, C, D, d, nnIndxLU, CIndx, n, theta, covModel, nThreads, fix_nugget);
+
+        for(int im = 0; im < sim_number; im++){
+            product_B_F(B, F, &sim[n*im], n, nnIndxLU, nnIndx, &sim_decor[n*im]);
+        }
+
+        //return stuff
+        SEXP result_r, resultName_r;
+        int nResultListObjs = 2;
+
+
+
+        PROTECT(result_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
+        PROTECT(resultName_r = allocVector(VECSXP, nResultListObjs)); nProtect++;
+
+        SET_VECTOR_ELT(result_r, 0, sim_r);
+        SET_VECTOR_ELT(resultName_r, 0, mkChar("sim"));
+
+        SET_VECTOR_ELT(result_r, 1, sim_decor_r);
+        SET_VECTOR_ELT(resultName_r, 1, mkChar("residual_sim"));
+
+        namesgets(result_r, resultName_r);
+
+        //unprotect
+        UNPROTECT(nProtect);
+
+
         return(result_r);
     }
 }
 
 
 
-
